#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#define FREE_ARG char*

void uerror(char error_text[])
/* standard error handler */
{
    fprintf(stderr, "Pointer utilities run-time error...\n");
    fprintf(stderr, "%s\n", error_text);
    exit(1);
}

unsigned long *lvector(long nl, long nh)
/* allocate an unsigned long vector with subscript range v[nl..nh] */
{
    unsigned long *v;

    v=(unsigned long *)malloc((size_t) ((nh-nl+1+1)*sizeof(long)));
    if (!v) uerror("allocation failure in lvector()");
    return v-nl+1;
}

int *ivector(long nl, long nh)
/* allocate an int vector with subscript range v[nl..nh] */
{
    int *v;
    
    v=(int *)malloc((size_t) ((nh-nl+1+1)*sizeof(int)));
    if (!v) uerror("allocation failure in ivector()");
    return v-nl+1;
}

double *dvector(long nl, long nh)
/* allocate a double vector with subscript range v[nl..nh] */
{
    double *v;

    v=(double *)malloc((size_t) ((nh-nl+1+1)*sizeof(double)));
    if(!v) uerror("allocation failure in dvector()");
    return v-nl+1;
}

double **dmatrix(long nrl, long nrh, long ncl, long nch)
/* allocate a double matrix with subscript range m[nrl..nrh][ncl..nch] */
{
    long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
    double **m;
    
    /* allocate pointers to rows */
    m=(double **)malloc((size_t) ((nrow+1)*sizeof(double*)));
    if (!m) uerror("allocation failure 1 in matrix()");
    m += 1;
    m-= nrl;

    /* allocate rows and set pointers to them */
    m[nrl]=(double *)malloc((size_t) ((nrow*ncol +1)*sizeof(double)));
    if(!m[nrl]) uerror("allocation failure 2 in matrix()");
    m[nrl] += 1;
    m[nrl] -= ncl;

    for(i=nrl+1; i<=nrh; i++) m[i]=m[i-1]+ncol;

    /* return pointer to array of pointers to rows */
    return m;
}

void free_ivector(int *v, long nl, long nh)
/* free an int vector allocated with ivector() */
{
    free((FREE_ARG) (v+nl-1));
}

void free_dvector(double *v, long nl, long nh)
/* free a double vector allocated with dvector() */
{
    free((FREE_ARG) (v+nl-1));
}

void free_lvector(unsigned long *v, long nl, long nh)
/* free an unsigned long vector allocated with lvector() */
{
    free((FREE_ARG) (v+nl-1));
}

void free_dmatrix(double **m, long nrl, long nrh, long ncl, long nch)
/* free a double matrix allocated by dmatrix() */
{
    free((FREE_ARG) (m[nrl]+ncl-1));
    free((FREE_ARG) (m+nrl-1));
}
